# Generated by Django 3.2.12 on 2022-07-06 10:13

import django.db.models.deletion
from django.db import (
    migrations,
    models,
)

import audoma.django.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ("audoma_api", "0004_merge_20220620_1037"),
    ]

    operations = [
        migrations.CreateModel(
            name="Country",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", audoma.django.db.fields.CharField(max_length=225)),
            ],
        ),
        migrations.RemoveField(
            model_name="exampletagmodel",
            name="item",
        ),
        migrations.DeleteModel(
            name="ExampleSimpleModel",
        ),
        migrations.DeleteModel(
            name="ExampleTagModel",
        ),
        migrations.AlterField(
            model_name="exampleperson",
            name="birth_country",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, to="audoma_api.country"
            ),
        ),
    ]
